"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
const buttonUtils = __importStar(require("./buttons"));
const algorithmx = __importStar(require("algorithmx"));
class CanvasModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign({}, super.defaults(), { _model_name: CanvasModel.model_name, _model_module: CanvasModel.model_module, _model_module_version: CanvasModel.model_module_version, _view_name: CanvasModel.view_name, _view_module: CanvasModel.view_module, _view_module_version: CanvasModel.view_module_version, _dispatch_events: [] });
    }
}
CanvasModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers);
CanvasModel.model_name = 'CanvasModel';
CanvasModel.model_module = version_1.MODULE_NAME;
CanvasModel.model_module_version = version_1.MODULE_VERSION;
CanvasModel.view_name = 'CanvasModel';
CanvasModel.view_module = version_1.MODULE_NAME;
CanvasModel.view_module_version = version_1.MODULE_VERSION;
exports.CanvasModel = CanvasModel;
class CanvasView extends base_1.DOMWidgetView {
    constructor() {
        super(...arguments);
        this.client = null;
        this.canvas = null;
        this.eventIndex = 0;
        this.stopped = false;
    }
    playEvents(events) {
        if (this.client === null)
            return;
        events.forEach(eventStr => {
            const event = JSON.parse(eventStr);
            this.client.dispatch(event);
        });
    }
    playAllEvents() {
        const events = this.model.get('_dispatch_events');
        this.playEvents(events);
    }
    eventsChanged() {
        if (this.client === null)
            return;
        const events = this.model.get('_dispatch_events');
        const newEvents = events.slice(this.eventIndex);
        this.eventIndex = events.length;
        this.playEvents(newEvents);
    }
    resetCanvas() {
        if (this.canvas === null)
            return;
        const immediateCanvas = this.canvas.eventQ(null).duration(0);
        immediateCanvas.cancelall().startall();
        immediateCanvas.remove();
        immediateCanvas.add().size([400, 250]).zoomkey(true);
        setTimeout(() => { immediateCanvas.svgattr('width', '100%'); }, 1);
    }
    removeCanvas() {
        this.resetCanvas();
        const element = this.el;
        const canvasDiv = element.querySelector('algorithmx-container');
        if (canvasDiv !== null)
            element.removeChild(canvasDiv);
        const buttonDiv = element.querySelector('algorithmx-buttons');
        if (buttonDiv !== null)
            element.removeChild(buttonDiv);
    }
    remove() {
        this.removeCanvas();
        super.remove();
    }
    clickRestart() {
        if (this.canvas === null)
            return;
        this.resetCanvas();
        this.playAllEvents();
        this.stopped = false;
    }
    clickStart() {
        if (this.canvas === null)
            return;
        this.canvas.eventQ(null).startall();
        this.stopped = false;
    }
    clickStop() {
        if (this.canvas === null)
            return;
        this.canvas.eventQ(null).stopall();
        this.stopped = true;
    }
    renderButtons() {
        const element = this.el;
        const buttonDiv = document.createElement('div');
        const buttonPlay = buttonUtils.createButton('pause', () => {
            if (this.stopped) {
                buttonUtils.setIcon(buttonPlay, 'pause');
                this.clickStart();
            }
            else {
                buttonUtils.setIcon(buttonPlay, 'play');
                this.clickStop();
            }
        });
        const buttonRestart = buttonUtils.createButton('repeat', () => {
            buttonUtils.setIcon(buttonPlay, 'pause');
            this.clickRestart();
        });
        buttonDiv.appendChild(buttonPlay);
        buttonDiv.appendChild(buttonRestart);
        element.appendChild(buttonDiv);
    }
    render() {
        this.removeCanvas();
        const canvasDiv = document.createElement('div');
        canvasDiv.setAttribute('id', 'algorithmx-container');
        this.client = algorithmx.client(canvasDiv);
        this.client.subscribe(event => {
            const fullEvent = { source: 'algorithmx', data: event };
            const fullEventStr = JSON.stringify(fullEvent);
            this.send(fullEventStr);
        });
        this.canvas = this.client.canvas();
        this.resetCanvas();
        const element = this.el;
        element.appendChild(canvasDiv);
        const showButtons = this.model.get('_show_buttons');
        if (showButtons)
            this.renderButtons();
        this.model.on('change:_dispatch_events', this.eventsChanged, this);
        this.eventsChanged();
    }
}
exports.CanvasView = CanvasView;
//# sourceMappingURL=widget.js.map